/* AnnotationLabel.java

	Purpose:
		
	Description:
		
	History:
		Mon Nov 11 12:40:29 CST 2019, Created by rudyhuang

Copyright (C) 2019 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import java.util.LinkedHashMap;
import java.util.Map;

import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.chart.util.MapsHelper;
import org.zkoss.json.JavaScriptValue;
import org.zkoss.lang.Generics;

/**
 * Options for annotation's labels.
 * Each label inherits options from the labelOptions object.
 * An option from the labelOptions can be overwritten by config for a specific label.
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 *
 * @author rudyhuang
 * @since 7.2.1.0
 */
public class AnnotationLabelOptions extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		align, allowOverlap, backgroundColor, borderColor, borderRadius,
		borderWidth, className, crop, distance, format, formatter, overflow,
		padding, point, shadow, shape, style, useHTML, verticalAlign, x, y
	}

	/**
	 * Returns the alignment of the annotation's label.
	 * If right, the right side of the label should be touching the point.
	 * <p>Default: "center"
	 */
	public String getAlign() {
		return getAttr(Attrs.align, "center").asString();
	}

	/**
	 * Sets the alignment of the annotation's label.
	 * If right, the right side of the label should be touching the point.
	 */
	public void setAlign(String align) {
		setAttr(Attrs.align, align, "center");
	}

	/**
	 * Returns whether to allow the annotation's labels to overlap.
	 * To make the labels less sensitive for overlapping, the padding can be set to 0.
	 * <p>
	 * Default: false.
	 */
	public boolean isAllowOverlap() {
		return getAttr(Attrs.allowOverlap, false).asBoolean();
	}
	/**
	 * Sets whether to allow the annotation's labels to overlap.
	 * To make the labels less sensitive for overlapping, the padding can be set to 0.
	 */
	public void setAllowOverlap(Boolean allowOverlap) {
		setAttr(Attrs.allowOverlap, allowOverlap, false);
	}

	/**
	 * Returns the background color or gradient for the annotation's label.
	 * <p>
	 * Default: rgba(0, 0, 0, 0.75)
	 */
	public Color getBackgroundColor() {
		return (Color) getAttr(Attrs.backgroundColor, new Color("rgba(0, 0, 0, 0.75)")).asValue();
	}

	/**
	 * Sets the background color or gradient for the annotation's label.
	 */
	public void setBackgroundColor(Color color) {
		setAttr(Attrs.backgroundColor, color, new Color("rgba(0, 0, 0, 0.75)"));
	}

	/**
	 * Sets the background color or gradient for the annotation's label.
	 */
	public void setBackgroundColor(String color) {
		setBackgroundColor(new Color(color));
	}

	/**
	 * Sets the background color or gradient for the annotation's label.
	 */
	public void setBackgroundColor(LinearGradient color) {
		setBackgroundColor(new Color(color));
	}

	/**
	 * Sets the background color or gradient for the annotation's label.
	 */
	public void setBackgroundColor(RadialGradient color) {
		setBackgroundColor(new Color(color));
	}

	/**
	 * Returns the border color for the annotation's label.
	 * <p>
	 * Default: "black"
	 */
	public Color getBorderColor() {
		return (Color) getAttr(Attrs.borderColor, "black").asValue();
	}

	/**
	 * Sets the border color for the annotation's label.
	 */
	public void setBorderColor(Color color) {
		setAttr(Attrs.borderColor, color);
	}

	/**
	 * Sets the border color for the annotation's label.
	 */
	public void setBorderColor(String color) {
		setBorderColor(new Color(color));
	}

	/**
	 * Returns the border radius in pixels for the annotation's label.
	 * <p>
	 * Default: 3
	 */
	public Number getBorderRadius() {
		return getAttr(Attrs.borderRadius, 3).asNumber();
	}

	/**
	 * Sets the border radius in pixels for the annotation's label.
	 */
	public void setBorderRadius(Number borderRadius) {
		setAttr(Attrs.borderRadius, borderRadius, 3);
	}

	/**
	 * Returns the border width in pixels for the annotation's label
	 * <p>
	 * Default: 1
	 */
	public Number getBorderWidth() {
		return getAttr(Attrs.borderWidth, 1).asNumber();
	}

	/**
	 * Sets the border width in pixels for the annotation's label
	 */
	public void setBorderWidth(Number borderWidth) {
		setAttr(Attrs.borderWidth, borderWidth, 1);
	}

	/**
	 * Returns class name for styling by CSS.
	 * <p>
	 * Default: ""
	 */
	public String getClassName() {
		return getAttr(Attrs.className, "").asString();
	}

	/**
	 * Sets class name for styling by CSS.
	 */
	public void setClassName(String className) {
		setAttr(Attrs.className, className, "");
	}

	/**
	 * Returns whether to to hide the annotation's label that is outside the plot area.
	 * <p>
	 * Default: false
	 */
	public boolean isCrop() {
		return getAttr(Attrs.crop, false).asBoolean();
	}

	/**
	 * Sets whether to hide the annotation's label that is outside the plot area.
	 */
	public void setCrop(boolean crop) {
		setAttr(Attrs.crop, crop, false);
	}

	/**
	 * Returns the label's pixel distance from the point.
	 * <p>
	 * Default: null
	 */
	public Number getDistance() {
		return getAttr(Attrs.distance, null).asNumber();
	}

	/**
	 * Sets the label's pixel distance from the point.
	 */
	public void setDistance(Number distance) {
		setAttr(Attrs.distance, distance);
	}

	/**
	 * Returns the <a href="https://www.highcharts.com/docs/chart-concepts/labels-and-string-formatting">format</a>
	 * string for the data label.
	 * <p>
	 * Default: null
	 */
	public String getFormat() {
		return getAttr(Attrs.format, null).asString();
	}

	/**
	 * Sets the <a href="https://www.highcharts.com/docs/chart-concepts/labels-and-string-formatting">format</a>
	 * string for the data label.
	 */
	public void setFormat(String format) {
		setAttr(Attrs.format, format);
	}

	/**
	 * Returns the callback JavaScript function to format the annotation's label.
	 * Note that if a format or text are defined, the format or text take precedence and the formatter is ignored.
	 * This refers to a point object.
	 * <p>
	 * Default: {@code function () { return defined(this.y) ? this.y : 'Annotation label'; }}
	 */
	public JavaScriptValue getFormatter() {
		return (JavaScriptValue) getAttr(Attrs.formatter, new JavaScriptValue("function () { return defined(this.y) ? this.y : 'Annotation label'; }")).asValue();
	}

	/**
	 * Sets the callback JavaScript function to format the annotation's label.
	 * Note that if a format or text are defined, the format or text take precedence and the formatter is ignored.
	 * This refers to a point object.
	 */
	public void setFormatter(JavaScriptValue formatter) {
		setAttr(Attrs.formatter, formatter);
	}

	/**
	 * Returns how to handle the annotation's label that flow outside the plot area.
	 * The justify option aligns the label inside the plot area.
	 * <p>
	 * Default: "justify".
	 */
	public String getOverflow() {
		return getAttr(Attrs.overflow, "justify").asString();
	}

	/**
	 * Sets how to handle the annotation's label that flow outside the plot area.
	 * The justify option aligns the label inside the plot area.
	 */
	public void setOverflow(String overflow) {
		setAttr(Attrs.overflow, overflow, "justify");
	}

	/**
	 * Returns the padding within the box when either the {@link #getBorderWidth()} or the
	 * {@link #getBackgroundColor()} is set.
	 * <p>
	 * Default: 5
	 */
	public Number getPadding() {
		return getAttr(Attrs.padding, 5).asNumber();
	}

	/**
	 * Sets the padding within the box when either the {@link #getBorderWidth()} or the
	 * {@link #getBackgroundColor()} is set.
	 */
	public void setPadding(int padding) {
		setAttr(Attrs.padding, padding, 5);
	}

	/**
	 * Returns the point which the label will be connected.
	 * It can be either the point which exists in the series - it is referenced by the point's id
	 * - or a new point with defined x, y properties and optionally axes.
	 */
	public Object getPoint() {
		return getAttr(Attrs.point);
	}

	/**
	 * Sets the point which the label will be connected.
	 */
	public void setPoint(Annotation.Point point) {
		setAttr(Attrs.point, point);
	}

	/**
	 * Sets the id of the point which the label will be connected.
	 */
	public void setPoint(String pointId) {
		setAttr(Attrs.point, pointId);
	}

	/**
	 * Returns whether to apply a drop shadow to the outer chart area. Requires
	 * that backgroundColor be set.
	 * <p>
	 * Default: false.
	 *
	 * @see Shadow
	 */
	public boolean isShadow() {
		return getShadow() != Shadow.NONE;
	}

	/**
	 * Returns whether to apply a shadow of the box.
	 * <p>
	 * Default: {@link Shadow#NONE}.
	 * @see Shadow
	 */
	public Shadow getShadow() {
		return (Shadow) getAttr(Attrs.shadow, Shadow.NONE).asValue();
	}

	/**
	 * Sets whether to apply a shadow of the box.
	 */
	public void setShadow(boolean shadow) {
		setShadow(shadow ? new Shadow() : Shadow.NONE);
	}

	/**
	 * Sets whether to apply a shadow of the box.
	 * @see Shadow
	 */
	public void setShadow(Shadow shadow) {
		setAttr(Attrs.shadow, shadow);
	}

	/**
	 * Returns the name of a symbol to use for the border around the label.
	 * Symbols are predefined functions on the Renderer object.
	 * <p>Default: callout
	 */
	public String getShape() {
		return getAttr(Attrs.shape, "callout").asString();
	}

	/**
	 * Sets the name of a symbol to use for the border around the label.
	 * Symbols are predefined functions on the Renderer object.
	 */
	public void setShape(String shape) {
		setAttr(Attrs.shape, shape, "callout");
	}

	/**
	 * Returns CSS styles for the annotation's label.
	 * <p>
	 * Default: empty
	 * @param <K> type for the key
	 * <p>
	 * if style was set using {@link #setStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 * @param <V> type for the value
	 * <p>
	 * if style was set using {@link #setStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 */
	public <K, V> Map<K, V> getStyle() {
		if (!containsKey(Attrs.style)) {
			setStyle("");
		}
		return Generics.cast(getAttr(Attrs.style));
	}

	/**
	 * Sets CSS styles for the annotation's label.
	 * <p>
	 * This method allows <b>simple</b> settings like
	 * <pre>{@code
	 * label.setStyle("fontWeight: 'bold'; color: 'white'; textShadow: '0px 1px 2px black'");
	 * }</pre>
	 * <p>
	 * In the format of CSS key:value, and ";" separating multiple key:value pairs,
	 * if you have more a complex CSS styling, we recommend using {@link #setStyle(Map)}
	 * @param style one or many CSS key:value pairs in a single line string format
	 */
	public void setStyle(String style) {
		setStyle(MapsHelper.parse(new LinkedHashMap<String, String>(), style, ':', ';', '\''));
	}

	/**
	 * Sets CSS styles for the annotation's label.
	 * <p>
	 * This method allows <b>complex</b> styling like:
	 * <pre>
	 * {@code
	 * Map<String, Object> styles = new HashMap<String, Object>();
	 * styles.put("fill", "#E8E8EA");
	 * styles.put("stroke", "#6E6E78");
	 * styles.put("stroke-width", "1");
	 * styles.put("r", "0");
	 * Map<String, Object> states = new HashMap<String, Object>();
	 * Map<String, String> hover = new HashMap<String, String>();
	 * hover.put("fill", "#F4F4F6");
	 * hover.put("stroke", "#6E6E78");
	 * hover.put("stroke-width", "1");
	 * hover.put("r", "0");
	 * states.put("hover", hover);
	 * styles.put("states", states);
	 * Map<String, String> style = new HashMap<String, String>();
	 * style.put("color", "#4D5663");
	 * style.put("fontSize", "12px");
	 * style.put("fontFamily", "'Arial'");
	 * style.put("fontWeight", "normal");
	 * style.put("padding", "10px");
	 * styles.put("style", style);
	 * }</pre>
	 * which will generate the final results like:
	 * <pre>{@code
	 * {
	 *     "fill": "#E8E8EA",
	 *     "stroke": "#6E6E78",
	 *     "stroke-width": "1",
	 *     "r": "0",
	 *     "states":
	 *     {
	 *         "hover":
	 *         {
	 *             "fill": "#F4F4F6",
	 *             "stroke": "#6E6E78",
	 *             "stroke-width": "1",
	 *             "r": "0"
	 *         }
	 *     },
	 *     "style":
	 *     {
	 *         "color": "#4D5663",
	 *         "fontSize": "12px",
	 *         "fontFamily": "'Arial'",
	 *         "fontWeight": "normal",
	 *         "padding": "10px"
	 *     }
	 * }
	 * }</pre>
	 * @param <K> Type for the key, usually String
	 * @param <V> Type for the value, usually String. For nested maps, use Object or any other type.
	 * @param style the style object
	 */
	public <K, V> void setStyle(Map<K, V> style) {
		setAttr(Attrs.style, style, NOT_NULL_VALUE);
	}

	/**
	 * Alias for the format option.
	 */
	public String getText() {
		return getFormat();
	}

	/**
	 * Alias for the format option.
	 */
	public void setText(String text) {
		setFormat(text);
	}

	/**
	 * Returns whether to <a href="https://www.highcharts.com/docs/chart-concepts/labels-and-string-formatting#html">use
	 * HTML</a> to render the annotation's label.
	 * <p>
	 * Default: false.
	 */
	public boolean isUseHTML() {
		return getAttr(Attrs.useHTML, false).asBoolean();
	}

	/**
	 * Sets whether to <a href="https://www.highcharts.com/docs/chart-concepts/labels-and-string-formatting#html">use
	 * HTML</a> to render the annotation's label.
	 */
	public void setUseHTML(boolean useHTML) {
		setAttr(Attrs.useHTML, useHTML);
	}

	/**
	 * Returns the vertical alignment of the annotation's label.
	 * <p>Default: bottom
	 */
	public String getVerticalAlign() {
		return getAttr(Attrs.verticalAlign, "bottom").asString();
	}

	/**
	 * Sets the vertical alignment of the annotation's label.
	 */
	public void setVerticalAlign(String verticalAlign) {
		setAttr(Attrs.verticalAlign, verticalAlign, "bottom");
	}

	/**
	 * Returns the x position offset of the label relative to the point.
	 * Note that if a distance is defined, the distance takes precedence over x and y options.
	 * <p> Default: 0
	 */
	public Number getX() {
		return getAttr(Attrs.x, 0).asNumber();
	}

	/**
	 * Sets the x position offset of the label relative to the point.
	 */
	public void setX(Number x) {
		setAttr(Attrs.x, x, 0);
	}

	/**
	 * Returns the y position offset of the label relative to the point.
	 * Note that if a distance is defined, the distance takes precedence over x and y options.
	 * <p> Default: -16
	 */
	public Number getY() {
		return getAttr(Attrs.y, -16).asNumber();
	}

	/**
	 * Sets the y position offset of the label relative to the point.
	 */
	public void setY(Number y) {
		setAttr(Attrs.y, y, -16);
	}
}
