package org.zkoss.chart.plotOptions;

import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.util.DynamicalAttribute;

/**
 * An option for solid gauge plot.
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 * 
 * @author hanhsu
 */
public class SolidGaugePlotOptions extends GaugePlotOptions {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		innerRadius, linecap, overshoot, radius, rounded
	}

	/**
	 * Returns the inner radius for points in a solid gauge.
	 * Can be given as a number (pixels) or percentage string.
	 * <p>
	 * Default: 60
	 * @since 7.2.1.0
	 */
	public Object getInnerRadius() {
		return getAttr(Attrs.innerRadius, 60).asValue();
	}

	/**
	 * Sets the inner radius for points in a solid gauge.
	 * Can be given as a number (pixels).
	 * @since 7.2.1.0
	 */
	public void setInnerRadius(Number innerRadius) {
		setAttr(Attrs.innerRadius, innerRadius);
	}

	/**
	 * Sets the inner radius for points in a solid gauge.
	 * Can be given as a percentage string.
	 * @since 7.2.1.0
	 */
	public void setInnerRadius(String innerRadius) {
		setAttr(Attrs.innerRadius, innerRadius);
	}

	/**
	 * Returns whether the strokes of the solid gauge should be round or square.
	 * <p>
	 * Default: "round"
	 * @since 7.2.1.0
	 */
	public String getLinecap() {
		return getAttr(Attrs.linecap, "round").asString();
	}

	/**
	 * Sets whether the strokes of the solid gauge should be round or square.
	 * @since 7.2.1.0
	 */
	public void setLinecap(String linecap) {
		if (!("round".equals(linecap) || "square".equals(linecap)))
			throw new IllegalArgumentException("Unsupported linecap: [" + linecap + "]");
		setAttr(Attrs.linecap, linecap);
	}

	/**
	 * Allow the gauge to overshoot the end of the perimeter axis by this many degrees.
	 * Say if the gauge axis goes from 0 to 60, a value of 100, or 1000,
	 * will show 5 degrees beyond the end of the axis when this option is set to 5.
	 * <p>
	 * Default: 0
	 * @since 7.2.1.0
	 */
	public Number getOvershoot() {
		return getAttr(Attrs.overshoot, 0).asNumber();
	}

	/**
	 * Allow the gauge to overshoot the end of the perimeter axis by this many degrees.
	 * Say if the gauge axis goes from 0 to 60, a value of 100, or 1000,
	 * will show 5 degrees beyond the end of the axis when this option is set to 5.
	 * @since 7.2.1.0
	 */
	public void setOvershoot(Number overshoot) {
		setAttr(Attrs.overshoot, overshoot);
	}

	/**
	 * Returns the outer radius for points in a solid gauge.
	 * Can be given as a number (pixels) or percentage string.
	 * <p>
	 * Default: 100
	 * @since 7.2.1.0
	 */
	public Object getRadius() {
		return getAttr(Attrs.radius, 100).asValue();
	}

	/**
	 * Sets the outer radius for points in a solid gauge.
	 * Can be given as a number (pixels).
	 * @since 7.2.1.0
	 */
	public void setRadius(Number radius) {
		setAttr(Attrs.radius, radius);
	}

	/**
	 * Sets the outer radius for points in a solid gauge.
	 * Can be given as a percentage string.
	 * @since 7.2.1.0
	 */
	public void setRadius(String radius) {
		setAttr(Attrs.radius, radius);
	}

	/**
	 * Returns whether to draw rounded edges on the gauge.
	 * <p>
	 * Default: false
	 * @since 7.2.1.0
	 */
	public boolean isRounded() {
		return getAttr(Attrs.rounded, false).asBoolean();
	}

	/**
	 * Sets whether to draw rounded edges on the gauge.
	 * @since 7.2.1.0
	 */
	public void setRounded(boolean rounded) {
		setAttr(Attrs.rounded, rounded);
	}
}
